// Generated by gmmproc 2.86.0 -- DO NOT MODIFY!
#ifndef _GIOMM_SETTINGSSCHEMASOURCE_P_H
#define _GIOMM_SETTINGSSCHEMASOURCE_P_H


#include <glibmm/private/object_p.h>


#endif /* _GIOMM_SETTINGSSCHEMASOURCE_P_H */

